@@ -28,7 +28,7 @@ my @clocks;
 foreach my $name (@zones) {
     my $zone   = DateTime::TimeZone->new(name => $name);
     my $offset = int($zone->offset_for_datetime($now) / 60);
-    my $time   = $now->with_offset($offset);
+    my $time   = $now->with_offset_same_instant($offset);
     $name =~ s![^/]+/!!;
     $name =~ s!_!\x20!g;
     push @clocks, [$name, $time];
