@@ -21,32 +21,38 @@
 #  endif
 #endif
 
-#define SECS_PER_WEEK       604800
-#define SECS_PER_DAY        86400
-#define SECS_PER_HOUR       3600
-#define SECS_PER_MIN        60
-#define SECS_PER_NANO       1000000000
+#define SECS_PER_WEEK         604800
+#define SECS_PER_DAY          86400
+#define SECS_PER_HOUR         3600
+#define SECS_PER_MIN          60
+#define SECS_PER_MILLI        1000
+#define SECS_PER_MICRO        1000000
+#define SECS_PER_NANO         1000000000
 
-#define MIN_UNIT_YEARS      INT64_C(-10000)
-#define MAX_UNIT_YEARS      INT64_C(10000)
-#define MIN_UNIT_MONTHS     INT64_C(-120000)
-#define MAX_UNIT_MONTHS     INT64_C(120000)
-#define MIN_UNIT_WEEKS      INT64_C(-521775)
-#define MAX_UNIT_WEEKS      INT64_C(521775)
-#define MIN_UNIT_DAYS       INT64_C(-3652425)
-#define MAX_UNIT_DAYS       INT64_C(3652425)
-#define MIN_UNIT_HOURS      INT64_C(-87658200)
-#define MAX_UNIT_HOURS      INT64_C(87658200)
-#define MIN_UNIT_MINUTES    INT64_C(-5259492000)
-#define MAX_UNIT_MINUTES    INT64_C(5259492000)
-#define MIN_UNIT_SECONDS    INT64_C(-315569520000)
-#define MAX_UNIT_SECONDS    INT64_C(315569520000)
+#define MIN_UNIT_YEARS        INT64_C(-10000)
+#define MAX_UNIT_YEARS        INT64_C(10000)
+#define MIN_UNIT_MONTHS       INT64_C(-120000)
+#define MAX_UNIT_MONTHS       INT64_C(120000)
+#define MIN_UNIT_WEEKS        INT64_C(-521775)
+#define MAX_UNIT_WEEKS        INT64_C(521775)
+#define MIN_UNIT_DAYS         INT64_C(-3652425)
+#define MAX_UNIT_DAYS         INT64_C(3652425)
+#define MIN_UNIT_HOURS        INT64_C(-87658200)
+#define MAX_UNIT_HOURS        INT64_C(87658200)
+#define MIN_UNIT_MINUTES      INT64_C(-5259492000)
+#define MAX_UNIT_MINUTES      INT64_C(5259492000)
+#define MIN_UNIT_SECONDS      INT64_C(-315569520000)
+#define MAX_UNIT_SECONDS      INT64_C(315569520000)
+#define MIN_UNIT_MILLISECONDS INT64_C(-315569520000000)
+#define MAX_UNIT_MILLISECONDS INT64_C(315569520000000)
+#define MIN_UNIT_MICROSECONDS INT64_C(-315569520000000000)
+#define MAX_UNIT_MICROSECONDS INT64_C(315569520000000000)
 
-#define MIN_RANGE           INT64_C(86400)        /* 0001-01-01T00:00:00Z */
-#define MAX_RANGE           INT64_C(315537983999) /* 9999-12-31T23:59:59Z */
-#define UNIX_EPOCH          INT64_C(62135683200)  /* 1970-01-01T00:00:00Z */
-#define MIN_EPOCH_SEC       INT64_C(-62135596800) /* 0001-01-01T00:00:00Z */
-#define MAX_EPOCH_SEC       INT64_C(253402300799) /* 9999-12-31T23:59:59Z */
+#define MIN_RANGE             INT64_C(86400)        /* 0001-01-01T00:00:00Z */
+#define MAX_RANGE             INT64_C(315537983999) /* 9999-12-31T23:59:59Z */
+#define UNIX_EPOCH            INT64_C(62135683200)  /* 1970-01-01T00:00:00Z */
+#define MIN_EPOCH_SEC         INT64_C(-62135596800) /* 0001-01-01T00:00:00Z */
+#define MAX_EPOCH_SEC         INT64_C(253402300799) /* 9999-12-31T23:59:59Z */
 
 #define VALID_EPOCH_SEC(s) \
     (s >= MIN_EPOCH_SEC && s <= MAX_EPOCH_SEC)
@@ -65,6 +71,8 @@ typedef enum {
     MOMENT_UNIT_HOURS,
     MOMENT_UNIT_MINUTES,
     MOMENT_UNIT_SECONDS,
+    MOMENT_UNIT_MILLISECONDS,
+    MOMENT_UNIT_MICROSECONDS,
     MOMENT_UNIT_NANOSECONDS,
 } moment_unit_t;
 
@@ -72,10 +80,13 @@ typedef enum {
     MOMENT_COMPONENT_YEAR=0,
     MOMENT_COMPONENT_MONTH,
     MOMENT_COMPONENT_DAY_OF_YEAR,
+    MOMENT_COMPONENT_DAY_OF_QUARTER,
     MOMENT_COMPONENT_DAY_OF_MONTH,
     MOMENT_COMPONENT_HOUR,
     MOMENT_COMPONENT_MINUTE,
     MOMENT_COMPONENT_SECOND,
+    MOMENT_COMPONENT_MILLISECOND,
+    MOMENT_COMPONENT_MICROSECOND,
     MOMENT_COMPONENT_NANOSECOND,
 } moment_component_t;
 
@@ -98,8 +109,8 @@ dt_t        moment_local_dt(const moment_t *mt);
 void        moment_to_utc_rd_values(const moment_t *mt, IV *rdn, IV *sod, IV *nos);
 void        moment_to_local_rd_values(const moment_t *mt, IV *rdn, IV *sod, IV *nos);
 
-IV          moment_compare(const moment_t *m1, const moment_t *m2);
-IV          moment_compare_local(const moment_t *m1, const moment_t *m2);
+int         moment_compare(const moment_t *m1, const moment_t *m2);
+int         moment_compare_local(const moment_t *m1, const moment_t *m2);
 
 int         moment_year(const moment_t *mt);
 int         moment_quarter(const moment_t *mt);
